#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include "pcf8591.h"

#define MAIN_C
#include "exitfail.h"

#define BASENAME(p)   ((strrchr((p), '/') ? : ((p) - 1)) + 1)

#define REF_VOLTAGE	3.3	/* 基準電圧[V] */
#define I2C_ADDR	0x48	/* PCF8591のI2Cアドレス */
#define DEFAULT_CH	0	/* オプションで指定されなかった場合に使用するチャンネル */

static void usage(char *prog)
{
	printf("Usage: %s <-d|--device FILENAME> [-c|--channel CHANNEL]\n", BASENAME(prog));
}

static void parse_arg(int argc, char *argv[], char **device, int *ch)
{
	int c;
	char *endptr;

	*device = NULL;

	for(;;) {
		int option_index = 0;
		static struct option long_options[] = {
			/* name,        has_arg,           flag, val*/
			{"device",      required_argument, NULL, 'd'},
			{"channel",     required_argument, NULL, 'c'},
			{0,             0,                 0,    0},
		};

		c = getopt_long(argc, argv, "d:c:",
				long_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
		case 'd':
			*device = optarg;
			break;
		case 'c':
			errno = 0;
			*ch = strtol(optarg, &endptr, 0);
			if (errno != 0 || optarg == endptr)
				goto err;

			if (*ch < PCF8591_CH_MIN || PCF8591_CH_MAX < *ch)
				goto err;
			break;
		default:
			goto err;
		}
	}

	if (*device == NULL)
		goto err;

	return;

err:
	usage(argv[0]);
	exit(EXIT_FAILURE);
}

int main(int argc, char *argv[])
{
	struct pcf8591 *adc;
	char *device;
	int channel = DEFAULT_CH;
	double voltage;
	uint8_t digital_code;
	int ret;

	exitfail_init();

	parse_arg(argc, argv, &device, &channel);

	adc = pcf8591_open(device, I2C_ADDR);
	if (adc == NULL)
		exitfail_errno("pcf8591_open");

	ret = pcf8591_read(adc, channel, &digital_code);
	if (ret != 0)
		exitfail_errno("pcf8591_read");

	voltage = digital_code * REF_VOLTAGE /
	          ((1 << PCF8591_RESOLUTION_BITS) - 1);

	printf("%1.3fV\n", voltage);

	ret = pcf8591_close(adc);
	if (ret != 0)
		exitfail_errno("pcf8591_close");

	return EXIT_SUCCESS;
}
